﻿using Borepin.Model;
using System;
using System.Collections.Generic;
using System.Text;

namespace Borepin.Service
{
    public static class BFFHDataService
    {
        public static List<Machine> _Machines;
        public static List<User> _Users;

        static BFFHDataService()
        {
            _Users = new List<User>()
            {
                new User()
                {
                    ID = "0",
                    Name = "Nick Wise (Admin)",
                    OriginalWorkshop = "fvm.fab-access.org"
                },
                new User()
                {
                    ID = "1",
                    Name = "Tamia Langley",
                    OriginalWorkshop = "fvm.fab-access.org"
                },
                new User()
                {
                    ID = "2",
                    Name = "Jay James",
                    OriginalWorkshop = "fvm.fab-access.org"
                },
                new User()
                {
                    ID = "3",
                    Name = "Darragh Ellwood",
                    OriginalWorkshop = "happylab.fab-access.org"
                },
                new User()
                {
                    ID = "4",
                    Name = "Mila-Rose Mcknight",
                    OriginalWorkshop = "motionlab.fab-access.org"
                }
            };

            _Machines = new List<Machine>()
            {
                new Machine()
                {
                    ID = "MakerRobo UP",
                    Description = "MakerRobo UP is a 3D Printer from Bangladesh",
                    State = MachineStates.Free,
                    User = null
                },
                new Machine()
                {
                    ID = "LS-1080-K 150W",
                    Description = "Laser cutting and engraving" + "\n" +
                                  "Cutting Area: 1000 x 800 mm" + "\n" +
                                  "Materials: wood, cardboard, PMMA, textiles, etc." + "\n" +
                                  "Supported files: DXF, AI, EPS, JPG, BMP",
                    State = MachineStates.InUse,
                    User = _Users.Find(x => x.ID == "1")
                },
                new Machine()
                {
                    ID = "Fireball V90",
                    Description = "The V90 is a high performance, general purpose machine, suitable for many uses such as clock making, plaques, RC aircraft parts and more. 12x18x3 working space.",
                    State = MachineStates.ToCheck,
                    User = _Users.Find(x => x.ID == "0")
                },
                new Machine()
                {
                    ID = "IRB 1600-145 kg",
                    Description = "Robotic Arm",
                    State = MachineStates.Blocked,
                    User = null
                },
                new Machine()
                {
                    ID = "Carmine 1.09",
                    Description = "Primesense",
                    State = MachineStates.Disabled,
                    User = null
                },
                new Machine()
                {
                    ID = "Zmorph 2.0SX Multitool",
                    Description = "FabLab in a box ",
                    State = MachineStates.Reserved,
                    User = _Users.Find(x => x.ID == "2")
                },
                new Machine()
                {
                    ID = "CubePro Duo",
                    Description = "The CubePro features the largest-in-class build platform with ultra high-resolution. With prints 2.5 times larger than any other desktop prosumer and hobbyist printer (11.2 x 10.6 x 9.06 or 285.4mm x 270.4mm x 230mm) with ultra high-resolution settings of 70-micron thin print layers, professional quality printing has never been so large or easy.",
                    State = MachineStates.Free,
                    User = null
                },
            };
        }
    }

    public class BFFHInterface
    {
        public readonly User ActiveUser;
        public BFFHInterface(string userID)
        {
            ActiveUser = BFFHDataService._Users.Find(x => x.ID == userID);
        }

        public void UseMachine(string machineID)
        {
            Machine machinetouse = BFFHDataService._Machines.Find(x => x.ID == machineID);
            
            if (machinetouse == null)
            {
                throw new Exception("Unknown Machine");
            }

            int index = BFFHDataService._Machines.IndexOf(machinetouse);

            if (machinetouse.State == MachineStates.Free)
            {
                machinetouse.State = MachineStates.InUse;
                machinetouse.User = ActiveUser;
            }

            if(machinetouse.State == MachineStates.Reserved && machinetouse.User == ActiveUser)
            {
                machinetouse.State = MachineStates.InUse;
                machinetouse.User = ActiveUser;
            }

            BFFHDataService._Machines[index] = machinetouse;
        }

        public void ReserveMachine(string machineID)
        {
            Machine machinetouse = BFFHDataService._Machines.Find(x => x.ID == machineID);
            if (machinetouse == null)
            {
                throw new Exception("Unknown Machine");
            }

            int index = BFFHDataService._Machines.IndexOf(machinetouse);

            if (machinetouse.State == MachineStates.Free)
            {
                machinetouse.State = MachineStates.Reserved;
                machinetouse.User = ActiveUser;
            }

            BFFHDataService._Machines[index] = machinetouse;
        }

        public void GiveBackMachine(string machineID)
        {
            Machine machinetogiveback = BFFHDataService._Machines.Find(x => x.ID == machineID);
            if (machinetogiveback == null)
            {
                throw new Exception("Unknown Machine");
            }

            int index = BFFHDataService._Machines.IndexOf(machinetogiveback);

            if (machinetogiveback.State == MachineStates.InUse)
            {
                machinetogiveback.State = MachineStates.Free;
                machinetogiveback.User = null;
            }

            BFFHDataService._Machines[index] = machinetogiveback;
        }

        public void SetStateForce(string machineID, MachineStates state)
        {
            if(IsAdmin())
            {
                Machine machinetosetstate = BFFHDataService._Machines.Find(x => x.ID == machineID);
                if (machinetosetstate == null)
                {
                    throw new Exception("Unknown Machine");
                }

                int index = BFFHDataService._Machines.IndexOf(machinetosetstate);

                machinetosetstate.State = state;

                BFFHDataService._Machines[index] = machinetosetstate;
            }
            else
            {
                throw new Exception("Forbidden");
            }
        }

        public bool IsAdmin()
        {
            return ActiveUser.ID == BFFHService.AdminID;
        }
    }


    public static class BFFHService
    {
        public static readonly string Host = "fvm.fab-access.org";
        public static readonly string AdminID = "0";

        public static BFFHInterface Auth(string userID)
        {
            if(BFFHDataService._Users.Find(x => x.ID == userID) == null)
            {
                throw new Exception("Authentication failed");
            }

            return new BFFHInterface(userID);
        }

        public static List<Machine> GetMachines()
        {
            return BFFHDataService._Machines;
        }

        public static List<User> GetUsers()
        {
            return BFFHDataService._Users;
        }

        public static Machine GetMachine(string machineID)
        {
            Machine m = BFFHDataService._Machines.Find(x => x.ID == machineID);

            if (m == null)
            {
                throw new Exception("Unknown Machine");
            }

            return m;
        }
    }
}
